import React, {Component} from "react"
import {Redirect, Link} from "react-router-dom"
import axios from "axios"

import {SERVER_HOST} from "../config/global_constants"


export default class Register extends Component
{
    constructor(props)
    {
        super(props)
        
        this.state = {
            name:"",
            email:"",
            password:"",
            confirmPassword:"",    
            isRegistered:false
        } 
    }
    
    
    handleChange = (e) => 
    {
        this.setState({[e.target.name]: e.target.value})
    }
    
    
    handleSubmit = (e) => 
    {
        e.preventDefault()

        axios.defaults.withCredentials = true // needed for sessions to work
        axios.post(`${SERVER_HOST}/users/register/${this.state.name}/${this.state.email}/${this.state.password}`)
        .then(res => 
        {     
            if(res.data)
            {
                if (res.data.errorMessage)
                {
                    console.log(res.data.errorMessage)    
                }
                else // user successfully registered
                { 
                    console.log("User registered and logged in")
                    
                    sessionStorage.name = res.data.name
                    sessionStorage.accessLevel = res.data.accessLevel
                    
                    this.setState({isRegistered:true})
                }        
            }
            else
            {
                console.log("Registration failed")
            }
        })   
    }


    validateName()
    {    
        // valid name pattern
        return (this.state.name.length > 1)
    }
    
    
    validateEmail()
    {    
        // valid email pattern
        const pattern = /^(([^<>()\[\]\\.,:\s@"]+(\.[^<>()\[\]\\.,:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/  //eslint-disable-line
        return pattern.test(String(this.state.email).toLowerCase())
    }


    validatePassword()
    {    
        const pattern = /^(?=.*[0-9])(?=.*[a-z])(?=.*[A-Z])(?=.*[£!#€$%^&*]).{10,}$/   //eslint-disable-line
        return pattern.test(String(this.state.password)) 
    }


    validateConfirmPassword()
    {    
        return ((this.state.confirmPassword.length > 0) && (this.state.password === this.state.confirmPassword)) 
    }


    validate() 
    {
        return {
            name: this.validateName(),
            email: this.validateEmail(),
            password: this.validatePassword(),
            confirmPassword: this.validateConfirmPassword()
        }
    }


    render() 
    {
        
        const checkMark = "✔"
        const formInputsState = this.validate()
        const inputsAreAllValid = Object.keys(formInputsState).every(index => formInputsState[index])          
            
        let nameErrorMessage = "";
        let emailErrorMessage = "";
        let passwordErrorMessage = "";
        let confirmPasswordErrorMessage = "";

        if(!this.validateName())
        {
            nameErrorMessage = <div className="error">Name must be at least two characters long<br/></div>;
        }
        
        if(!this.validateEmail())
        {
            emailErrorMessage = <div className="error">Enter a valid email<br/></div>;
        }
        
        if(!this.validatePassword())
        {
            let errorMessages = [];
            if(this.state.password.length < 10)
            {
                errorMessages.push("Password must be at least ten characters long.");
            } 
            if(!this.state.password.match(/[0-9]/))
            {
                errorMessages.push("Password must contain at least one digit (0-9).");
            }
            if(!this.state.password.match(/[a-z]/))
            {
                errorMessages.push("Password must contain at least one lowercase character.");
            }
            if(!this.state.password.match(/[A-Z]/))
            {
                errorMessages.push("Password must contain at least one uppercase character.{'\n''}");
            }
            if(!this.state.password.match(/[£!#€$%^&*]/))
            {
                errorMessages.push("Password must contain at least one of the characters £!#€$%^&*");
            }           
            passwordErrorMessage = <div className="error"><ul>{errorMessages.map(errorMessage => <li key={errorMessage}> {errorMessage} </li>)}</ul></div>;                    
        }      
        
        if(!this.validateConfirmPassword())
        {
            confirmPasswordErrorMessage = <div className="error">Passwords must match<br/></div>;        
        }

            
        return (
            <form className="form-container" noValidate = {true} id = "loginOrRegistrationForm" onSubmit={this.handleSubmit}>
           
                {this.state.isRegistered ? <Redirect to="/DisplayAllCars"/> : null} 
            
                <h2>New User Registration</h2>
            
                <div id="nameContainer">
                    <input  
                        name = "name"              
                        type = "text"
                        placeholder = "Name"
                        autoComplete="name"
                        value = {this.state.name}
                        onChange = {this.handleChange}
                        ref = {(input) => { this.inputToFocus = input }} 
                    />
	           <span className={!this.validateName() ?"" : "showCorrectTick"}>{checkMark}</span>
		</div>
                {nameErrorMessage}
            
                <div id="emailContainer">
	            <input  
                        name = "email"              
                        type = "email"
                        placeholder = "Email"
                        autoComplete="email"
                        value = {this.state.email}
                        onChange = {this.handleChange}
                    />
		    <span className={!this.validateEmail() ?"" : "showCorrectTick"}>{checkMark}</span>
		</div>
               {emailErrorMessage}
               
                <div id="passwordContainer">
	            <input  
                        name = "password"           
                        type = "password"
                        placeholder = "Password"
                        autoComplete="password"
                        title = "Password must be at least ten-digits long and contains at least one lowercase letter, one uppercase letter, one digit and one of the following characters (£!#€$%^&*)"
                        value = {this.state.password}
                        onChange = {this.handleChange}
                    />
                    <span className={!this.validatePassword() ?"" : "showCorrectTick"}>{checkMark}</span>
                </div>
                {passwordErrorMessage}
            
                <div id="passwordContainer">
                    <input          
                        name = "confirmPassword"    
                        type = "password"
                        placeholder = "Confirm password"
                        autoComplete="confirmPassword"
                        value = {this.state.confirmPassword}
                        onChange = {this.handleChange}
                    />
                    <span className={!this.validateConfirmPassword() ?"" : "showCorrectTick"}>{checkMark}</span>
                </div>
                {confirmPasswordErrorMessage}
                          
                <input 
                    type = "submit" 
                    value = "Register" 
                    disabled = {!inputsAreAllValid}
                />
                
                <Link className="red-button" to={"/DisplayAllCars"}>Cancel</Link>   
            </form>
        )
    }
}